/* Jean Philippe EIMER - 2013 *
 *  phil.eimer@9online.fr     */

#include <math.h>

#include <iostream>
#include <iomanip>

#include "include.h"


//using namespace std;

void	Ephemerides::InitAstres(void)
{
	unsigned short int i;

//	maxastr=Orbite::PLU;
	maxastr=Orbite::NEP;	// pluto is not a planet anymore (since 2006)
	maxastr++;

	Obliquite();		// calcul de l'obliquité (epsilon)

	Soleil.Init(this, Orbite::SOL);
	Soleil.EcliptGeo();


	Lune.Init(this);
	Lune.EcliptGeo();

	Nutat.Calcule(this);	// calcul de la nutation

	Soleil.EcliptGeo2Horizon(Observ.Inst.t);
	Soleil.Diametre();
	Soleil.Magnitude();
	Soleil.LeverCoucher();


	Lune.EcliptGeo2Horizon(Observ.Inst.t);
	Lune.Diametre();
	Lune.Phase();
	Lune.Elongation();
	Lune.Age();
	Lune.LeverCoucher();


	for( i=0 ; i < maxastr ; i++ )
		Planete[i].Init(this,i);

	for( i=0 ; i < maxastr ; i++ )
	{
		Planete[i].EcliptGeo();

		Planete[i].EcliptGeo2Horizon(Observ.Inst.t);
		Planete[i].Diametre();
		Planete[i].Phase();
		Planete[i].Elongation();
		Planete[i].Magnitude();
		Planete[i].LeverCoucher();
	}

	Planete[Orbite::TER].Init(this, Orbite::TER);
}


void	Ephemerides::Obliquite(void)
{
	VectData	Veps=TER_EPS;


	epsilon=Veps.DotT(Observ.Inst.t-1.0);
}


void	Ephemerides::Affiche(wxEpheFrameOL *frame, BOOL_HMS hms)
{
	double	eps=epsilon;


	Frame = frame;
	Frame->Obliquity->SetLabel(wxString::Format(wxT("%7.4f"), Deci2Sexa(eps)));

	AfficheCoord(HELIO,hms);
	AfficheCoord(GEO,hms);
	AfficheCoord(GEOAPP,hms);
	AfficheCoord(EQUA,hms);
	AfficheCoord(HORAIRE,hms);
	AfficheCoord(HORIZON,hms);
	AfficheAPP();
	AfficheLMC();
}


void	Ephemerides::AfficheCoord(COORD_TYPE coord, BOOL_HMS hms)
{
	wxGrid	*grid;


	switch(coord)
	{
		case	HELIO:
			grid = Frame->GridEcliptHelio;
			break;

		case	GEO:
			grid = Frame->GridEcliptGeo;
			break;

		case	GEOAPP:
			grid = Frame->GridEcliptGeoApp;
			break;

		case	EQUA:
			grid = Frame->GridEquat;
			break;

		case	HORAIRE:
			grid = Frame->GridHour;
			break;

//		case	HORIZON:
		default:
			grid = Frame->GridHoriz;
			break;
	}

	if(coord<HORAIRE)
		AfficheLigneCoord(grid, wxT("%8.5f"), coord, Vecteur::COORD_R, DECI);
	AfficheLigneCoord(grid, wxT("%8.4f"), coord, Vecteur::COORD_TETA, hms);
	AfficheLigneCoord(grid, wxT("%8.4f"), coord, Vecteur::COORD_PHI, hms);
}


void	Ephemerides::AfficheLigneCoord(wxGrid *grid, const wxString &format, COORD_TYPE coord, Vecteur::COORD_POL coorditem, BOOL_HMS hms)
{
	unsigned short int i;
	double	co,dint;
	bool	heure=(coord==EQUA || coord==HORAIRE) && coorditem==Vecteur::COORD_TETA;
	unsigned int row = (unsigned int)coorditem;


	if(coord > EQUA)
		row--;

	// Soleil ou Terre
	if(coord==HELIO)
		co=Planete[Orbite::TER].Coord[coord].Spher[coorditem];
	else
		co=Soleil.Coord[coord].Spher[coorditem];

	if(heure)
		co/=KH2D;
	if(hms)
		co=Deci2Sexa(co);

	grid->SetCellValue(row , 0, wxString::Format(format, co) );

	// Lune
	co=Lune.Coord[coord].Spher[coorditem];

	if(heure)
		co/=KH2D;
	if(hms)
		co=Deci2Sexa(co);

	if(coord==GEO && coorditem==Vecteur::COORD_R)
	{
		co=Lune.parallaxe*60.0;
		co=modf(co,&dint)*60.0/100.0;
		co+=dint;
		grid->SetCellValue(row , 1, wxString::Format(wxT("P (\'\") %5.2f"), co));
	}
	else
		grid->SetCellValue(row , 1, wxString::Format(format, co) );

	// Planetes
	for(i=0;i<maxastr;i++)
	{
		co=Planete[i].Coord[coord].Spher[coorditem];

		if(heure)
			co/=KH2D;

		if(hms)
			co=Deci2Sexa(co);

		grid->SetCellValue(row , i+2, wxString::Format(format, co) );
	}
}


void	Ephemerides::AfficheAPP(void)
{
	unsigned short int	i;


	// phase
	Frame->GridApp->SetCellValue(0 , 1, wxString::Format(wxT("%5.2f"), Lune.phase));

	// phase en %
	Frame->GridApp->SetCellValue(1 , 1, wxString::Format(wxT("%5.2f"), Lune.phase100));

	// élongation
	Frame->GridApp->SetCellValue(2 , 1, wxString::Format(wxT("%5.2f"), Lune.elongation));

	// diamètre apparent
	Frame->GridApp->SetCellValue(3 , 0, wxString::Format(wxT("%2u\'%02u\""), (unsigned int)Soleil.diametre/60, (unsigned int)Soleil.diametre%60));
	Frame->GridApp->SetCellValue(3 , 1, wxString::Format(wxT("%2u\'%02u\""), (unsigned int)Lune.diametre/60, (unsigned int)Lune.diametre%60));

	// magnitude, âge pour la Lune
	i=(unsigned short int)Lune.age;
	Frame->GridApp->SetCellValue(4 , 0, wxString::Format(wxT("%5.2f"), Soleil.magnitude));
	Frame->GridApp->SetCellValue(4 , 1, wxString::Format(wxT("Age %2uj%02uh"), i, (unsigned short int)((Lune.age-i)*24.0)));

	for( i = 0 ; i < maxastr ; i++ )
	{
		Frame->GridApp->SetCellValue(0 , i+2, wxString::Format(wxT("%5.2f"), Planete[i].phase));
		Frame->GridApp->SetCellValue(1 , i+2, wxString::Format(wxT("%5.2f"), Planete[i].phase100));
		Frame->GridApp->SetCellValue(2 , i+2, wxString::Format(wxT("%5.2f"), Planete[i].elongation));
		Frame->GridApp->SetCellValue(3 , i+2, wxString::Format(wxT("%5.2f"), Planete[i].diametre));
		Frame->GridApp->SetCellValue(4 , i+2, wxString::Format(wxT("%5.2f"), Planete[i].magnitude));
	}
}


void	Ephemerides::AfficheLMC(void)
{
	unsigned short int	i;


	// levers
	AfficheHeureLMC(Soleil.Lever, 0, 0);
	AfficheHeureLMC(Lune.Lever, 0, 1);

	// passages au méridien
	AfficheHeureLMC(Soleil.Meridien, 1, 0);
	AfficheHeureLMC(Lune.Meridien, 1, 1);

	// couchers
	AfficheHeureLMC(Soleil.Coucher, 2, 0);
	AfficheHeureLMC(Lune.Coucher, 2, 1);

	AfficheDureesLMC(Soleil.Lever, Soleil.Coucher, 0);
	AfficheDureesLMC(Lune.Lever, Lune.Coucher, 1);

	for( i=0 ; i < maxastr ; i++ )
	{
		AfficheHeureLMC(Planete[i].Lever, 0, i+2);
		AfficheHeureLMC(Planete[i].Meridien, 1, i+2);
		AfficheHeureLMC(Planete[i].Coucher, 2, i+2);

		AfficheDureesLMC(Planete[i].Lever, Planete[i].Coucher, i+2);
	}
}


void	Ephemerides::AfficheHeureLMC(Instant &it, unsigned char row, unsigned char col)
{
	if( !it.IsValid() )
		Frame->GridLMC->SetCellValue(row , col, wxT("-"));								// no valid time
	else if(row%2)
		Frame->GridLMC->SetCellValue(row , col, it.dtlocal.Format(wxS("%H:%M:%S"), it.tz));		// meridian crossing time displayed with seconds
	else
		Frame->GridLMC->SetCellValue(row , col, it.dtlocal.Format(wxS("%H:%M"), it.tz));		// rising/setting times displayed without seconds
}


void	Ephemerides::AfficheDureesLMC(Instant &il, Instant &ic, unsigned char col)
{
	wxTimeSpan	ts;

	if( il.IsValid() && ic.IsValid() )
	{
		ts = (ic.dtlocal - il.dtlocal).Abs();
		Frame->GridLMC->SetCellValue(3 , col, ts.Format(wxS("%H:%M")));
		Frame->GridLMC->SetCellValue(4 , col, (wxTimeSpan(24, 0, 0, 0) - ts).Format(wxS("%H:%M")));
	}
	else
	{
		Frame->GridLMC->SetCellValue(3 , col, wxT("-"));
		Frame->GridLMC->SetCellValue(4 , col, wxT("-"));
	}
}
