/* Jean Philippe EIMER - 2013 *
 *  phil.eimer@9online.fr     */

#include <math.h>
#include <wchar.h>

#include <iostream>
#include <iomanip>
using namespace std;

#include "include.h"



double	Instant::Init(double longitude)
{
	wxDateTime::TimeZone	timezone = wxDateTime::Local;


	/* deal with strange - buggy - wxDateTime time zone management */
	if( dtlocal.IsDST() )
		timezone = wxDateTime::TimeZone( timezone.GetOffset() + 3600 );

	tzoffset = wxDateTime::TimeZone( timezone.GetOffset() - tz.GetOffset() );

	datejulienne = Julien( dtlocal, &tu );

	dj2t();
	Sideral(longitude);

	return(datejulienne);
}


double	Instant::tu2t(const Instant &Idate, const bool dst, double tuext)
{
	if(tuext == TUSING)
	{
		dtlocal.Set(0.0);
		datejulienne=0.0;
		tu = TUSING;
		t=0;
	}
	else
	{
		dtlocal = Idate.dtlocal;
		dtlocal.MakeUTC( !dst );

		dtlocal.SetHour( (unsigned char)tuext );
		tuext -= dtlocal.GetTm().hour;
		tuext *= 60.0;
		dtlocal.SetMinute( (unsigned char)tuext );
		tuext -= dtlocal.GetTm().min;
		dtlocal.SetSecond( (unsigned char)(tuext*60.0) );

		dtlocal.MakeFromUTC( !dst );

		datejulienne=Julien(dtlocal,&tu);
		dj2t();
	}

	return(t);
}


double	Instant::Julien(const wxDateTime &dtt, double *phdec)
{
	double	dj;


	dj = dtt.GetJulianDayNumber();

	dj += (double)tzoffset.GetOffset() / 3600.0 / 24.0;

	if(phdec!=NULL)
	{
		/* deal with strange - buggy - wxDateTime time zone management */
		if( tz.GetOffset()== tzoffset.GetOffset() )
			*phdec=hDeci( dtt.FromTimezone(tzoffset).GetTm(wxDateTime::UTC) );
		else
			*phdec=hDeci( dtt.GetTm(tzoffset.GetOffset()) );
	}

	return( dj );
}


double	Instant::InvJulien(wxDateTime &dtt, double dj)
{
	double	hdec,dint;
	wxDateTime::Tm	Tm;


	dtt.Set( dj );
	Tm = dtt.GetTm();

	hdec = modf(dj-0.5,&dint)*24.0;

	Deci2Sexa(hdec, (short int *)&Tm.hour, (unsigned char *)&Tm.min, (unsigned char *)&Tm.sec);

	return(hdec);
}


double	Instant::Sideral(double longitude)
{
	double	vtsg0[4]=TSGZERO,t1;
	VectData	V;

	V=vtsg0;

	t1=(floor(datejulienne-0.5)+0.5-DJ0)/36525.0-1.0;	// t-1 à 0hTU

	tsg0=V.DotT(t1);
	tsg0+=(t1-((double)dtlocal.GetTm(wxDateTime::UTC).year-2000.0)/100.0)*2400.0;
	tsg0=K24(tsg0);

	tsg=tsg0+tu*SOL2SID;
	tsg=K24(tsg);

	ts=tsg-longitude/KH2D;
	ts=K24(ts);

	return(ts);
}



double	Instant::hDeci(const wxDateTime::Tm &Tm) const
{
	double	hdec;


	hdec = Tm.hour + (Tm.min + Tm.sec/60.0)/60.0;
	hdec = K24(hdec);

	return(hdec);
}


double	Instant::dj2t(void)
{
	t=(datejulienne-DJ0)/36525.0;

	return(t);
}


bool	Instant::IsValid(void)
{
	return( tu != TUSING );
}


void	Instant::Affiche(wxEpheFrameOL *Frame) const
{
	Frame->LocalDate->SetValue(dtlocal);
	Frame->LocalTime->ChangeValue(dtlocal.FormatTime());
//	Frame->TimeZoneText->SetLabel(dtlocal.Format(wxT("%Z"), tz));
	Frame->TimeZoneText->SetLabel( wxString::Format( wxT("%+.4ld"), tz.GetOffset()/36 ) );

//	Frame->UTDateTime->SetLabel(dtlocal.Format(wxDefaultDateTimeFormat, wxDateTime::UTC));

	/* deal with strange - buggy - wxDateTime time zone management */
	if( tz.GetOffset()== tzoffset.GetOffset() )
		Frame->UTDateTime->SetLabel( dtlocal.FromTimezone(tzoffset).Format(wxDefaultDateTimeFormat, wxDateTime::UTC) );
	else
		Frame->UTDateTime->SetLabel( dtlocal.Format(wxDefaultDateTimeFormat, tzoffset ) );

	Frame->JulianDate->SetLabel(wxString::Format( wxT("%.4f"), datejulienne));

	Frame->G0ST->SetLabel(wxString::Format( wxT("%7.4f"), Deci2Sexa(tsg0)));
	Frame->GST->SetLabel(wxString::Format( wxT("%7.4f"), Deci2Sexa(tsg)));
	Frame->ST->SetLabel(wxString::Format( wxT("%7.4f"), Deci2Sexa(ts)));
}
