/* Jean Philippe EIMER - 2014 *
 *  phil.eimer@9online.fr     */

#include <math.h>
#include <wx/textfile.h>

#include "include.h"


#define	TER_RT_M	(TER_RT*1000.0)
#define	MAX_TZ	(12 * 3600L)


float	LitSexa(wxString &line, const wxUniChar sep = wxT(' ') )
{
	double	l0, l;
	float	result;


	line.ToDouble(&l0);
	result = fabs(l0);

	line = line.AfterFirst( sep );
	line.ToDouble(&l);
	result += l/60.0;

	line = line.AfterFirst( sep );
	line.ToDouble(&l);
	result += l/3600.0;

	result = copysign(result, l0);

	return(result);
}


inline	long	zone_with_dst( long z, bool dst )
{
	if( dst )
		z += 3600;

	if( z > MAX_TZ )
		z -= 2*MAX_TZ;

	if( z < -MAX_TZ )
		z += 2*MAX_TZ;

	return( z );
}


bool	Observateur::ChargeLieu(void)
{
	wxTextFile	*file;
	wxString filename;
	wxString line, word;
	bool	ret = false;
	double	li;


	if( !wxGetEnv(wxT("HOME"), &filename ) )
		filename = wxT(".");

#ifndef __WXMSW__
	filename.Append( wxT("/.epherc") );
#else
	filename.Append( wxT("/epherc") );
#endif

	file = new wxTextFile(filename);

	if( file->Exists() && file->Open() )
	{
		line = file->GetFirstLine();

		while( !file->Eof() && line.GetChar(0) == wxT('#') )		// filter comments
			line = file->GetNextLine();

		param.location_name = line.BeforeFirst( wxT('\t') );

		line = line.AfterFirst( wxT('\t') );

		word = line.BeforeFirst( wxT('\t') );
		param.location_long = LitSexa(word);

		line = line.AfterFirst( wxT('\t') );

		word = line.BeforeFirst( wxT('\t') );
		param.location_lat = LitSexa(word);

		line = line.AfterFirst( wxT('\t') );

		word = line.BeforeFirst( wxT('\t') );
		word.ToDouble(&li);
		param.location_alt = (int)li;

		line = line.AfterFirst( wxT('\t') );

		word = line.BeforeFirst( wxT('\t') );
		word.ToDouble(&li);
		param.location_zone = (long)(li * 3600.0);

		param.refraction = 0.61;
		eta1 = RAD2DEG(acos(TER_RT_M/(TER_RT_M+param.location_alt)));
		eta2L = RAD2DEG(0);	// atan(h/d);
		eta2C = RAD2DEG(0);	// atan(h/d);

		ret = true;
	}

	delete(file);

	return( ret );
}


bool Observateur::ChargeLieuXML(void)
{
	wxXmlDocument	xml;
	wxXmlNode	*child;
	wxString	filename, s1, s2;
	long	l;
	double	d;
	unsigned char	cpt = 0;
	bool	ret = false;


	if( !wxGetEnv(wxT("HOME"), &filename ) )
		filename = wxT(".");

#ifndef __WXMSW__
	filename.Append( wxT("/.wxepherc") );
#else
	filename.Append( wxT("/wxepherc") );
#endif

	if( wxTextFile( filename ).Exists() &&  xml.Load( filename ) && xml.GetRoot()->GetName() == wxT("wxEpheRC") )
	{
		child = xml.GetRoot()->GetChildren();

		while( child )
		{
			s1 = child->GetName();
			s2 = child->GetNodeContent();
			s2.ToLong(&l, 10);
			s2.ToDouble(&d);

			if ( s1 == wxT("location_name") )
			{
				param.location_name = s2;
				cpt++;
			}
			else if ( s1 == wxT("location_longitude") )
			{
				param.location_long = LitSexa( s2, wxT(':') );
				cpt++;
			}
			else if ( s1 == wxT("location_latitude") )
			{
				param.location_lat = LitSexa( s2, wxT(':') );
				cpt++;
			}
			else if ( s1 == wxT("location_altitude") )
			{
				param.location_alt = (int)l;
				cpt++;
			}
			else if ( s1 == wxT("location_time_zone") )
			{
				param.location_zone = (long)(d * 3600.0);
				cpt++;
			}

			else if ( s1 == wxT("refraction") )
			{
				param.refraction = (float)d;
				cpt++;
			}

			child = child->GetNext();
		}	// while child

		if( cpt != PARAMETER_NB )
			wxLogMessage( wxString::Format( _("%u missing parameter(s) !"), PARAMETER_NB - cpt) );
		else
		{
			eta1 = RAD2DEG(acos(TER_RT_M/(TER_RT_M+param.location_alt)));
			eta2L = RAD2DEG(0);	// atan(h/d);
			eta2C = RAD2DEG(0);	// atan(h/d);
			ret = true;
		}
	}	// if xml.load

	return( ret );
}


void	Observateur::InitLieu(const wxString &n, double lng, double lat, int alt, long z)
{
	param.location_name = wxString(n);
	param.location_long = lng;
	param.location_lat = lat;
	param.location_alt = alt;
	param.location_zone = z;

	param.refraction = 0.61;
	eta1 = RAD2DEG(acos(TER_RT_M/(TER_RT_M+param.location_alt)));
	eta2L = RAD2DEG(0);	// atan(h/d);
	eta2C = RAD2DEG(0);	// atan(h/d);
}



double	Observateur::InitTemps(void)
{
	zone = zone_with_dst( param.location_zone, dst );
	Inst.tz = wxDateTime::TimeZone( zone );

	Inst.dtlocal.SetToCurrent();
	Inst.dtlocal.SetMillisecond(0);

	Inst.Init(param.location_long);

	return(Inst.tu);
}

double	Observateur::InitTemps( wxDateTime &wxdt, long z )
{
	zone = zone_with_dst( z, dst );
	Inst.tz = wxDateTime::TimeZone( zone );

	Inst.dtlocal = wxdt;
	Inst.dtlocal.SetMillisecond(0);

	Inst.Init(param.location_long);

	return(Inst.tu);
}


/*
double	Observateur::InitTemps(unsigned char jour, unsigned char mois, int annee, unsigned char heure, unsigned char min, unsigned char sec, bool btu)
{
	if(btu)		// heure entrée en TU
	{
		Inst.dtlocal.Set(jour, (wxDateTime::Month)(mois-1), annee, heure, min, sec, 0);
		Inst.dtlocal.FromUTC();
	}
	else		// heure entrée comme heure locale
	{
		Inst.dtlocal.Set(jour, (wxDateTime::Month)(mois-1), annee, heure, min, sec, 0);
	}
	Inst.tz = wxDateTime::Local;

	Inst.Init(longitude, zone, btu);

	return(Inst.tu);
}
*/

void	Observateur::Affiche(wxEpheFrameOL *Frame) const
{
	Frame->ObsLocation->SetLabel(param.location_name);
	Frame->Longitude->SetLabel(wxString::Format( wxT("%s %8.4f"), (param.location_long>0?_(L"W"):_(L"E")), fabs(Deci2Sexa(param.location_long))));
	Frame->Latitude->SetLabel(wxString::Format( wxT("%s %8.4f"), (param.location_lat>0?_(L"N"):_(L"S")), fabs(Deci2Sexa(param.location_lat))));
	Frame->Altitude->SetLabel(wxString::Format( wxT("%u"), param.location_alt));
	Frame->TimeZoneValue->ChangeValue( wxString::Format( wxT("%+.4ld"), (zone-(dst?3600:0))/36 ) );
}
