#ifndef _ORBITE_HPP_
#define _ORBITE_HPP_
class	Orbite
{
	Orbite	operator =(const double [6][4]);
	void	PerturbationsSoleil(double);	// calcul des perturbations pour le soleil
	void	PerturbationsTellur(double);	// perturbations pour Mercure, Vénus et Mars
	void	PerturbationsGeantes(double);	// perturbations pour les planètes géantes
	double	AnomalieMoyenne(void);	// calcul de M et omega
	double	Kepler(void);		// équation de Kepler (calcul de E)
	double	AnomalieVraie(void);	// calcul de r et v


	public:
		typedef	enum {MER,VEN,MAR,JUP,SAT,URA,NEP,PLU,TER,SOL,LUN} PLANETE;

		Astre	*pA;	// pour l'accès à tous les éléments de l'astre correspondant

		// Vecteurs d'initialisation des éléments orbitaux
		VectData	Ve;
		VectData	Vi;
		VectData	VL;
		VectData	VO;
		VectData	Vob;

		// Eléments calculés
		double	a;	// demi grand axe
		double	e;	// excentricité
		double	i;	// inclinaison sur l'écliptique
		double	L;	// longitude moyenne
		double	Omega;	// longitude du noeud ascendant
		double	omegab;	// longitude du périhélie
		double	deltaL;	// variation élémentaire de L, pour la prise en compte du temps de trajet de la lumière

		// Perturbations calculées
		double	aper;	// demi grand axe
		double	eper;	// excentricité
		double	Lper;	// longitude moyenne
		double	vkper;	// longitude du périhélie (associée avec e)
		double	rper;	// rayon vecteur
		double	lper;	// longitude
		double	bper;	// latitude

		// Eléments calculés d'après les précédents
		double	M;	// anomalie moyenne = L-omegab
		double	MRsp;	// anomalie moyenne sans perturbations, en radians
		double	omega;	// argument de latitude du périhélie = omegab-Omega

		double	E,ER;	// anomalie excentrique (obtenue par Kepler) en degrès et radians

		double	r;	// rayon vecteur
		double	v;	// anomalie vraie


		void	Init(Astre *);
		Orbite	operator =(const Orbite &);
		void	Elements(double);	// initialisation des éléments
		void	Perturbations(double);	// initialisation des perturbations
		void	Anomalies(void);

//		void	Affiche(void) const;		// affichage
};


class	OrbiteLune
{
	public:
		AstreLune	*pA;

		VectData	VO,VL,VM,VF,VD;				// éléments de l'orbite
		VectData	VLper,VMper,VFper,VDper;	// pour les perturbations
		VectData	Va,Vb,Vc,Ve;

		double		L,M,F,D;
		double		Omega,OmegaR,CCR,EE;


		void	Init(AstreLune *);
		void	Elements(const Instant &);
};
#endif	// _ORBITE_HPP_
